# SEVERAL WAYS TO PASS INFORMATION TO A FUNCTION USED BY NLM, INTEGERATE, ETC.

# As an illustration, we'll write a function to find the minimum of (x-a)^2 
# with respect to x, using nlm, where a is a constant that we wish to set to 
# various values in different calls of the function (to 3.1 and -1.2 in the
# examples).  We'll always use 0 as the initial value for the minimization.


# ONE GOOD WAY, USING THE EXTRA ARGUMENTS THAT NLM PASSES ON.

minf1 <- function (my_a) {
  nlm (function (x,a) (x-a)^2, 0, my_a) $ estimate
  #                               ^^^^ <- this gets passed on as argument a
}

cat ("minf1:", minf1(3.1), minf1(-1.2), "\n")


# ANOTHER GOOD WAY, REFERRING TO THE ARGUMENT OF MINF2 FROM THE LOCAL FUNCTION.

minf2 <- function (my_a) {
  nlm (function (x) (x-my_a)^2, 0) $ estimate
  #                    ^^^^ <- this refers to the my_a argument of minf2
}

cat ("minf2:", minf2(3.1), minf2(-1.2), "\n")

# or the same but with the local function given a name...

minf3 <- function (my_a) {
  f <- function (x) (x-my_a)^2
  #                    ^^^^ <- this refers to the my_a argument of minf3
  nlm (f, 0) $ estimate
}

cat ("minf3:", minf3(3.1), minf3(-1.2), "\n")


# A BAD WAY, USING A GLOBAL VARIABLE.

f <- function (x) (x-my_a)^2
#                  ^^^^ <- this refers to the global variable called my_a

minf4 <- function () {
  nlm (f, 0) $ estimate
}

cat ("minf4:")
my_a <- 3.1        # we specify the value of a by setting a global variable
cat ("",minf4())   #   ... this is both inconvenient and prone to error
my_a <- -1.2
cat ("",minf4(),"\n")


# A VERY BAD WAY, USING A GLOBAL VARIABLE, BUT PRETENDING TO USE AN ARGUMENT.

f <- function (x) (x-my_a)^2
#                  ^^^^ <- this refers to the global variable called my_a

minf5 <- function (my_a) {
#                  ^^^^ <- this argument is never used for anything
  nlm (f, 0) $ estimate
}

cat ("minf5:")
my_a <- 3.1
cat ("",minf5(3.1)) # the argument of minf5 is actually irrelevant
my_a <- -1.2
cat ("",minf5(-1.2),"\n")

cat ("minf5 (2):")
my_a <- 3.1
cat ("",minf5(123)) # ... as we can see here, the argument 123 is ignored!
my_a <- -1.2
cat ("",minf5(456),"\n")

